@@ -18,7 +18,7 @@ class RequestContext(Context):
         self.query: DictEx = self.cms.query
         self.form: DictEx = self.cms.form
         self.__headers: dict = None
-        self.responce_type: ResponseTypes = ResponseTypes.RENDERED
+        self.response_type: ResponseTypes = ResponseTypes.RENDERED
         self.status_code: HttpStatusCodes = HttpStatusCodes.OK
         self.mime = HttpMimeTypes.HTML
 
@@ -32,18 +32,18 @@ class RequestContext(Context):
         else:
             self.__headers[key].append(value)
 
-    def generate_error_responce(self, exception: Exception) -> dict:
-        """Generate error responce from process result"""
-        error_object = self._generate_error_object(exception)
+    def generate_error_response(self, exception: Exception) -> dict:
+        """Generate error response from process result"""
+        error_object, self.status_code = self._generate_error_object(exception)
         content = f"{error_object['errorMessage']} (Error Code: {error_object['errorCode']})"
-        return self.generate_responce(content)
+        return self.generate_response(content)
 
-    def generate_responce(self, result: Any) -> dict:
-        """Generate responce from process result"""
+    def generate_response(self, result: Any) -> dict:
+        """Generate response from process result"""
 
         ret_val = self.cms
         ret_val[HttpBaseDataType.CMS][HttpBaseDataName.WEB_SERVER] = {
-            "index": self.responce_type.value,
+            "index": self.response_type.value,
             "headercode": self.status_code.value,
             "mime": self.mime
         }
