@@ -1,4 +1,5 @@
 from bclib.dispatcher.dispatcher import Dispatcher
-from bclib.dispatcher.dispatcher_interface import IDispatcher
+from bclib.dispatcher.idispatcher import IDispatcher
 from bclib.dispatcher.socket_dispatcher import SocketDispatcher
 from bclib.dispatcher.dev_server_dispatcher import DevServerDispatcher
+from bclib.dispatcher.routing_dispatcher import RoutingDispatcher
