@@ -1,17 +1,20 @@
 """Base class for dispaching request"""
 import asyncio
-from abc import ABC, abstractmethod
-from typing import Callable, Any
+import inspect
+from abc import ABC
+import signal
+from typing import Callable, Any, Coroutine
 from functools import wraps
 
+from bclib.logger import ILogger, LoggerFactory
 from bclib.cache import create_chaching
-from bclib.listener import RabbitBusListener, MessageType
-from bclib.predicate import Predicate, InList, Equal, Url, Between, NotEqual, GreaterThan, LessThan, LessThanEqual, GreaterThanEqual, Match, HasValue, Callback
+from bclib.listener import RabbitBusListener
+from bclib.predicate import Predicate
 from bclib.context import ClientSourceContext, ClientSourceMemberContext, WebContext, Context, RESTfulContext, RabbitContext, SocketContext, ServerSourceContext, ServerSourceMemberContext
 from bclib.db_manager import DbManager
 from bclib.utility import DictEx
-from ..dispatcher.callback_info import CallbackInfo
 from bclib.exception import HandlerNotFoundErr
+from ..dispatcher.callback_info import CallbackInfo
 
 
 class Dispatcher(ABC):
@@ -21,8 +24,10 @@ class Dispatcher(ABC):
         self.options = DictEx(options)
         self.__look_up: 'dict[str, list[CallbackInfo]]' = dict()
         cache_options = self.options.cache if "cache" in self.options else None
+        self.event_loop = asyncio.get_event_loop()
         self.cache_manager = create_chaching(cache_options)
         self.db_manager = DbManager(self.options)
+        self.__logger: ILogger = LoggerFactory.create(self.options)
         self.__rabbit_dispatcher: 'list[RabbitBusListener]' = list()
         if "router" in self.options and "rabbit" in self.options.router:
             for setting in self.options.router.rabbit:
@@ -32,11 +37,19 @@ class Dispatcher(ABC):
     def socket_action(self, * predicates: (Predicate)):
         """Decorator for determine Socket action"""
 
-        def _decorator(socket_action_handler: 'Callable[[SocketContext],None]'):
+        def _decorator(socket_action_handler: 'Callable[[SocketContext],bool]'):
+
             @wraps(socket_action_handler)
-            def wrapper(context: SocketContext):
-                socket_action_handler(context)
-                return True
+            async def non_async_wrapper(context: SocketContext):
+                return socket_action_handler(context)
+
+            @wraps(socket_action_handler)
+            async def async_wrapper(context: SocketContext):
+                return await socket_action_handler(context)
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                socket_action_handler) else non_async_wrapper
+
             self._get_context_lookup(SocketContext.__name__)\
                 .append(CallbackInfo([*predicates], wrapper))
             return socket_action_handler
@@ -46,9 +59,20 @@ class Dispatcher(ABC):
         """Decorator for determine RESTful action"""
 
         def _decorator(restful_action_handler: 'Callable[[RESTfulContext], dict]'):
+
             @wraps(restful_action_handler)
-            def wrapper(context: RESTfulContext):
-                return context.generate_responce(restful_action_handler(context))
+            async def non_async_wrapper(context: RESTfulContext):
+                action_result = restful_action_handler(context)
+                return None if action_result is None else context.generate_response(action_result)
+
+            @wraps(restful_action_handler)
+            async def async_wrapper(context: RESTfulContext):
+                action_result = await restful_action_handler(context)
+                return None if action_result is None else context.generate_response(action_result)
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                restful_action_handler) else non_async_wrapper
+
             self._get_context_lookup(RESTfulContext.__name__)\
                 .append(CallbackInfo([*predicates], wrapper))
             return restful_action_handler
@@ -58,9 +82,20 @@ class Dispatcher(ABC):
         """Decorator for determine legacy web request action"""
 
         def _decorator(web_action_handler: 'Callable[[WebContext], str]'):
+
             @wraps(web_action_handler)
-            def wrapper(context: WebContext):
-                return context.generate_responce(web_action_handler(context))
+            async def non_async_wrapper(context: WebContext):
+                action_result = web_action_handler(context)
+                return None if action_result is None else context.generate_response(action_result)
+
+            @wraps(web_action_handler)
+            async def async_wrapper(context: WebContext):
+                action_result = await web_action_handler(context)
+                return None if action_result is None else context.generate_response(action_result)
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                web_action_handler) else non_async_wrapper
+
             self._get_context_lookup(WebContext.__name__)\
                 .append(CallbackInfo([*predicates], wrapper))
             return web_action_handler
@@ -71,14 +106,14 @@ class Dispatcher(ABC):
 
         def _decorator(client_source_action_handler: 'Callable[[ClientSourceContext], Any]'):
             @wraps(client_source_action_handler)
-            def wrapper(context: ClientSourceContext):
+            async def non_async_wrapper(context: ClientSourceContext):
                 data = client_source_action_handler(context)
                 result_set = list()
                 if data is not None:
                     for member in context.command.member:
                         member_context = ClientSourceMemberContext(
                             context, data, member)
-                        dispath_result = self.dispatch(member_context)
+                        dispath_result = await self.dispatch_async(member_context)
                         result = {
                             "options": {
                                 "tableName": member_context.table_name,
@@ -89,15 +124,51 @@ class Dispatcher(ABC):
                             "data": dispath_result
                         }
                         result_set.append(result)
-                ret_val = {
-                    "setting": {
-                        "keepalive": False,
-                    },
-                    "sources": result_set
-                }
-                return context.generate_responce(ret_val)
+                    ret_val = {
+                        "setting": {
+                            "keepalive": False,
+                        },
+                        "sources": result_set
+                    }
+                    return context.generate_response(ret_val)
+                else:
+                    return None
+
+            @wraps(client_source_action_handler)
+            async def async_wrapper(context: ClientSourceContext):
+                data = await client_source_action_handler(context)
+                result_set = list()
+                if data is not None:
+                    for member in context.command.member:
+                        member_context = ClientSourceMemberContext(
+                            context, data, member)
+                        dispath_result = await self.dispatch_async(member_context)
+                        result = {
+                            "options": {
+                                "tableName": member_context.table_name,
+                                "keyFieldName": member_context.key_field_name,
+                                "statusFieldName": member_context.status_field_name,
+                                "mergeType": member_context.merge_type.value
+                            },
+                            "data": dispath_result
+                        }
+                        result_set.append(result)
+                    ret_val = {
+                        "setting": {
+                            "keepalive": False,
+                        },
+                        "sources": result_set
+                    }
+                    return context.generate_response(ret_val)
+                else:
+                    return None
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                client_source_action_handler) else non_async_wrapper
+
             self._get_context_lookup(ClientSourceContext.__name__)\
                 .append(CallbackInfo([*predicates], wrapper))
+
             return client_source_action_handler
         return _decorator
 
@@ -105,8 +176,20 @@ class Dispatcher(ABC):
         """Decorator for determine source member action methode"""
 
         def _decorator(client_source_member_handler: 'Callable[[ClientSourceMemberContext], Any]'):
+
+            @wraps(client_source_member_handler)
+            async def non_async_wrapper(context: WebContext):
+                return client_source_member_handler(context)
+
+            @wraps(client_source_member_handler)
+            async def async_wrapper(context: WebContext):
+                return await client_source_member_handler(context)
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                client_source_member_handler) else non_async_wrapper
+
             self._get_context_lookup(ClientSourceMemberContext.__name__)\
-                .append(CallbackInfo([*predicates], client_source_member_handler))
+                .append(CallbackInfo([*predicates], wrapper))
             return client_source_member_handler
         return _decorator
 
@@ -115,14 +198,14 @@ class Dispatcher(ABC):
 
         def _decorator(server_source_action_handler: 'Callable[[ServerSourceContext], Any]'):
             @wraps(server_source_action_handler)
-            def wrapper(context: ServerSourceContext):
+            async def non_async_wrapper(context: ServerSourceContext):
                 data = server_source_action_handler(context)
                 result_set = list()
                 if data is not None:
                     for member in context.command.member:
                         member_context = ServerSourceMemberContext(
                             context, data, member)
-                        dispath_result = self.dispatch(member_context)
+                        dispath_result = await self.dispatch_async(member_context)
                         result = {
                             "options": {
                                 "tableName": member_context.table_name,
@@ -133,15 +216,51 @@ class Dispatcher(ABC):
                             "data": dispath_result
                         }
                         result_set.append(result)
-                ret_val = {
-                    "setting": {
-                        "keepalive": False,
-                    },
-                    "sources": result_set
-                }
-                return context.generate_responce(ret_val)
+                    ret_val = {
+                        "setting": {
+                            "keepalive": False,
+                        },
+                        "sources": result_set
+                    }
+                    return context.generate_response(ret_val)
+                else:
+                    return None
+
+            @wraps(server_source_action_handler)
+            async def async_wrapper(context: ServerSourceContext):
+                data = await server_source_action_handler(context)
+                result_set = list()
+                if data is not None:
+                    for member in context.command.member:
+                        member_context = ServerSourceMemberContext(
+                            context, data, member)
+                        dispath_result = await self.dispatch_async(member_context)
+                        result = {
+                            "options": {
+                                "tableName": member_context.table_name,
+                                "keyFieldName": member_context.key_field_name,
+                                "statusFieldName": member_context.status_field_name,
+                                "mergeType": member_context.merge_type.value
+                            },
+                            "data": dispath_result
+                        }
+                        result_set.append(result)
+                    ret_val = {
+                        "setting": {
+                            "keepalive": False,
+                        },
+                        "sources": result_set
+                    }
+                    return context.generate_response(ret_val)
+                else:
+                    return None
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                server_source_action_handler) else non_async_wrapper
+
             self._get_context_lookup(ServerSourceContext.__name__)\
                 .append(CallbackInfo([*predicates], wrapper))
+
             return server_source_action_handler
         return _decorator
 
@@ -149,22 +268,43 @@ class Dispatcher(ABC):
         """Decorator for determine server source member action methode"""
 
         def _decorator(server_source_member_action_handler: 'Callable[[ServerSourceMemberContext], Any]'):
+
+            @wraps(server_source_member_action_handler)
+            async def non_async_wrapper(context: WebContext):
+                return server_source_member_action_handler(context)
+
+            @wraps(server_source_member_action_handler)
+            async def async_wrapper(context: WebContext):
+                return await server_source_member_action_handler(context)
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                server_source_member_action_handler) else non_async_wrapper
+
             self._get_context_lookup(ServerSourceMemberContext.__name__)\
-                .append(CallbackInfo([*predicates], server_source_member_action_handler))
+                .append(CallbackInfo([*predicates], wrapper))
             return server_source_member_action_handler
         return _decorator
 
     def rabbit_action(self, * predicates: (Predicate)):
         """Decorator for determine rabbit-mq message request action"""
 
-        def _decorator(rabbit_action_handler: 'Callable[[RabbitContext], None]'):
+        def _decorator(rabbit_action_handler: 'Callable[[RabbitContext], bool]'):
+
             @wraps(rabbit_action_handler)
-            def wrapper(context: RabbitContext):
-                rabbit_action_handler(context)
-                return True
+            async def non_async_wrapper(context: RabbitContext):
+                return rabbit_action_handler(context)
+
+            @wraps(rabbit_action_handler)
+            async def async_wrapper(context: RabbitContext):
+                return await rabbit_action_handler(context)
+
+            wrapper = async_wrapper if inspect.iscoroutinefunction(
+                rabbit_action_handler) else non_async_wrapper
+
             self._get_context_lookup(RabbitContext.__name__)\
                 .append(CallbackInfo([*predicates], wrapper))
-            return wrapper
+
+            return rabbit_action_handler
         return _decorator
 
     def _get_context_lookup(self, key: str) -> 'list[CallbackInfo]':
@@ -178,7 +318,7 @@ class Dispatcher(ABC):
             self.__look_up[key] = ret_val
         return ret_val
 
-    def dispatch(self, context: Context) -> Any:
+    async def dispatch_async(self, context: Context) -> Any:
         """Dispatch context and get result from related action methode"""
 
         result: Any = None
@@ -186,107 +326,38 @@ class Dispatcher(ABC):
         try:
             items = self._get_context_lookup(name)
             for item in items:
-                result = item.try_execute(context)
+                result = await item.try_execute_async(context)
                 if result is not None:
                     break
             else:
-                result = context.generate_error_responce(
+                result = context.generate_error_response(
                     HandlerNotFoundErr(name))
         except Exception as ex:
-            result = context.generate_error_responce(ex)
+            result = context.generate_error_response(ex)
         return result
 
-    def initialize_task(self, loop: asyncio.AbstractEventLoop):
+    def initialize_task(self):
         for dispacher in self.__rabbit_dispatcher:
-            dispacher.initialize_task(loop)
+            dispacher.initialize_task(self.event_loop)
 
     def listening(self):
-        for dispacher in self.__rabbit_dispatcher:
-            dispacher.start_listening()
+        """Start listening to request for process"""
+        for sig in (signal.SIGTERM, signal.SIGINT):
+            signal.signal(sig, lambda sig, _: self.event_loop.stop())
+        self.initialize_task()
+        self.event_loop.run_forever()
+        tasks = asyncio.all_tasks(loop=self.event_loop)
+        for task in tasks:
+            task.cancel()
+        group = asyncio.gather(*tasks, return_exceptions=True)
+        self.event_loop.run_until_complete(group)
+        self.event_loop.close()
 
-    def run_in_background(self, callback: Callable, *args: Any) -> Any:
-        """helper for run function in background thread"""
+    async def log_async(self, **kwargs):
+        """log params"""
+        await self.__logger.log_async(**kwargs)
 
-        loop = asyncio.get_event_loop()
-        return loop.run_in_executor(None, callback, *args)
-
-    @abstractmethod
-    async def send_message(self, message: MessageType) -> bool:
-        """Send message to endpoint"""
-
-    def cache(self, seconds: int = 0, key: str = None):
-        """Cache result of function for seconds of time or until signal by key for clear"""
-
-        return self.cache_manager.cache_decorator(seconds, key)
-
-    @staticmethod
-    def in_list(expression: str, *items) -> Predicate:
-        """Create list cheking predicate"""
-
-        return InList(expression,  *items)
-
-    @staticmethod
-    def equal(expression: str, value: Any) -> Predicate:
-        """Create equality cheking predicate"""
-
-        return Equal(expression, value)
-
-    @staticmethod
-    def url(pattern: str) -> Predicate:
-        """Create url cheking predicate"""
-
-        return Url(pattern)
-
-    @staticmethod
-    def between(expression: str, min_value: int, max_value: int) -> Predicate:
-        """Create between cheking predicate"""
-
-        return Between(expression, min_value, max_value)
-
-    @staticmethod
-    def not_equal(expression: str, value: Any) -> Predicate:
-        """Create not equality cheking predicate"""
-
-        return NotEqual(expression, value)
-
-    @staticmethod
-    def greater_than(expression: str, value: int) -> Predicate:
-        """Create not greater than cheking predicate"""
-
-        return GreaterThan(expression, value)
-
-    @staticmethod
-    def less_than(expression: str, value: int) -> Predicate:
-        """Create not less than cheking predicate"""
-
-        return LessThan(expression, value)
-
-    @staticmethod
-    def less_than_equal(expression: str, value: int) -> Predicate:
-        """Create not less than and equal cheking predicate"""
-
-        return LessThanEqual(expression, value)
-
-    @staticmethod
-    def greater_than_equal(expression: str, value: int) -> Predicate:
-        """Create not less than and equal cheking predicate"""
-
-        return GreaterThanEqual(expression, value)
-
-    @staticmethod
-    def match(expression: str, value: str) -> Predicate:
-        """Create regex matching cheking predicate"""
-
-        return Match(expression, value)
-
-    @staticmethod
-    def has_value(expression: str) -> Predicate:
-        """Create has value cheking predicate"""
-
-        return HasValue(expression)
-
-    @staticmethod
-    def callback(callback: 'Callable[[Context],bool]') -> Predicate:
-        """Create Callback cheking predicate"""
-
-        return Callback(callback)
+    def log_in_background(self, **kwargs) -> Coroutine:
+        """log params in background precess"""
+        return self.event_loop.create_task(
+            self.__logger.log_async(**kwargs))
