@@ -1,16 +1,20 @@
+import asyncio
 import json
 import re
 from struct import error
-from typing import Callable
+from typing import Callable, Any
+from abc import abstractmethod
 
-from bclib.utility.dict_ex import DictEx
+from bclib.utility import DictEx
 
 from bclib.context import ClientSourceContext, RESTfulContext, WebContext, RequestContext, Context, SocketContext, ServerSourceContext
 from bclib.listener import Message, MessageType, HttpBaseDataType
+
+from ..dispatcher.dispatcher_helper import DispatcherHelper
 from ..dispatcher.dispatcher import Dispatcher
 
 
-class RoutingDispatcher(Dispatcher):
+class RoutingDispatcher(Dispatcher, DispatcherHelper):
 
     def __init__(self, options: dict):
         super().__init__(options)
@@ -46,7 +50,7 @@ class RoutingDispatcher(Dispatcher):
                     for value in values:
                         if len(value.strip()) != 0 and value not in route_dict:
                             route_dict[value] = key
-        if len(route_dict) == 1 and '*' in route_dict:
+        if len(route_dict) == 1 and '*' in route_dict and self.__default_router is None:
             router = route_dict['*']
             self.__context_type_detector: 'Callable[[str],str]' = lambda _: router
         else:
@@ -69,18 +73,18 @@ class RoutingDispatcher(Dispatcher):
                 print("Error in detect context from routing options!", ex)
         return context_type if context_type else self.__default_router
 
-    async def _on_message_receive(self, message: Message) -> Message:
+    async def _on_message_receive_async(self, message: Message) -> Message:
         """Process received message"""
 
         try:
             context = self.__context_factory(message)
-            response = self.dispatch(context)
+            response = await self.dispatch_async(context)
             ret_val: Message = None
             if context.is_adhoc:
                 message_result = json.dumps(response).encode("utf-8")
                 ret_val = Message.create_add_hock(
                     message.session_id, message_result)
-                await self.send_message(ret_val)
+                await self.send_message_async(ret_val)
             return ret_val
         except error as ex:
             print(f"Error in process received message {ex}")
@@ -127,3 +131,18 @@ class RoutingDispatcher(Dispatcher):
             raise Exception(
                 f"Configured context type '{context_type}' not found for '{url}'")
         return ret_val
+
+    def run_in_background(self, callback: Callable, *args: Any) -> asyncio.Future:
+        """helper for run function in background thread"""
+
+        loop = asyncio.get_running_loop()
+        return loop.run_in_executor(None, callback, *args)
+
+    @abstractmethod
+    async def send_message_async(self, message: MessageType) -> bool:
+        """Send message to endpoint"""
+
+    def cache(self, seconds: int = 0, key: str = None):
+        """Cache result of function for seconds of time or until signal by key for clear"""
+
+        return self.cache_manager.cache_decorator(seconds, key)
