@@ -22,7 +22,7 @@ class RabbitBusListener(RabbitListener):
             }
             new_context = RabbitContext(
                 DictEx(message), self.__dispatcher)
-            self.__dispatcher.dispatch(new_context)
+            self.__dispatcher.dispatch_async(new_context)
         except error as ex:
             print(
                 f"error in dispatcher received message from rabbit in {self._host}:{self._queue_name} ({ex})")
